/*
===============================================================================

	FILE:	codec.cpp
	
	CONTENTS:
	
		Face Fixer - SIGGRAPH 2000 demonstration software
	
	PROGRAMMERS:
	
		Martin Isenburg@cs.unc.edu
	
	COPYRIGHT:
	
		Copyright (C) 2000  Martin Isenburg (isenburg@cs.unc.edu)
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
		28 April 2000 -- Martin - finalized for SIGGRAPH submission.
	
===============================================================================
*/
#include "codec.h"

#include "label.h"
#include "vector.h"
#include "twinedge.h"

FFcodec::FFcodec()
{
  ls = new FFvector();
  gs = new FFvector();
  r = 0;
  l = 0;
  s = 0;
  e = 0;
  h = 0;
  m = 0;
  for (int i = 0; i < 7; i++) f[i] = 0;
  large_f = 0;
}

FFcodec::~FFcodec()
{
  FFlabel* l;
  for (int i = 0; i < ls->size(); i++)
    {
      l = (FFlabel*) ls->elementAt(i);
      delete l;
    }
  
  delete ls;
  delete gs;
}

void FFcodec::pushLabelR()
{
  ls->addElement((void*) new FFlabel(FFlabel::R));
  r++;
}

void FFcodec::pushLabelL()
{
  ls->addElement((void*) new FFlabel(FFlabel::L));
  l++;
}

void FFcodec::pushLabelS()
{
  ls->addElement((void*) new FFlabel(FFlabel::S));
  s++;
}

void FFcodec::pushLabelE()
{
  ls->addElement((void*) new FFlabel(FFlabel::E));
  e++;
}

void FFcodec::pushLabelF(int d)
{
  ls->addElement((void*) new FFlabel(FFlabel::F, d));
  if (d < 7) f[d]++;
  else    large_f++;
}

void FFcodec::pushLabelH(int d)
{
  ls->addElement((void*) new FFlabel(FFlabel::H, d));
  h++;
}

void FFcodec::pushLabelM(int i, int o1, int o2)
{
  ls->addElement((void*) new FFlabel(FFlabel::M, i, o1, o2));
  m++;
}

void FFcodec::pushGate(FFtwinEdge* g)
{
  gs->addElement((void*)g);
}

FFlabel* FFcodec::popLabel()
{
  FFlabel *label = (FFlabel *) ls->lastElement();
  ls->removeLastElement();

  if (label->type == FFlabel::E)
    {
      e--;
    }
  else if (label->type == FFlabel::S)
    {
      s--;
    }
  else if (label->type == FFlabel::L)
    {
      l--;
    }
  else if (label->type == FFlabel::R)
    {
      r--;
    }
  else if (label->type == FFlabel::M)
    {
      m--;
    }
  else if (label->type == FFlabel::H)
    {
      h--;
    }
  else if (label->type == FFlabel::F)
    {
      if (label->getDegree() < 7) f[label->getDegree()]--;
      else                                      large_f--;
    }
  return label;
}

FFtwinEdge* FFcodec::popGate()
{
  FFtwinEdge* g = (FFtwinEdge*)gs->lastElement();
  gs->removeLastElement();
  return g;
}

int FFcodec::hasMoreGates()
{
  return (gs->isEmpty() == 0);
}
